@@ -2,7 +2,7 @@
 # similar to stats::weighted.mean, except that 0s in w are not
 # strong 0s, and na.rm = TRUE removes NAs in x and w
 
-.mean <- function(x, w, na.rm, scale) {
+.mean <- function(x, w, na.rm) {
   if (na.rm) {
     if (anyNA(x) || anyNA(w)) {
       keep <- !(is.na(x) | is.na(w))
@@ -10,11 +10,11 @@
       w <- w[keep]
     } 
   }
-  sum(w * x) / if (scale) sum(w) else 1
+  sum(w * x) / sum(w)
 }
 
 #---- Generalized mean ----
-mean_generalized <- function(r) {
+generalized_mean <- function(r) {
   if (!is_number(r)) {
     stop("'r' must be a finite length 1 numeric")
   }
@@ -22,33 +22,33 @@ mean_generalized <- function(r) {
     warning("'r' is very small in absolute value, but not zero; this can give misleading results")
   }
   # return function
-  function(x, w = rep(1, length(x)), na.rm = FALSE, scale = TRUE) {
+  function(x, w = rep(1, length(x)), na.rm = FALSE) {
     if (length(x) != length(w)) {
       stop("'x' and 'w' must be the same length")
     }
     if (any_negative(x, w)) {
-      warning("Some elements of 'x' or 'w' are less than or equal to 0; the generalized mean is not defined")
+      warning("some elements of 'x' or 'w' are less than or equal to 0; the generalized mean is not defined")
     }
     # this works more-or-less the same as genmean in StatsBase.jl
     if (r == 0) {
-      exp(.mean(log(x), w, na.rm, scale))
+      exp(.mean(log(x), w, na.rm))
     } else {
-      .mean(x %^% r, w, na.rm, scale)^(1 / r) 
+      .mean(x %^% r, w, na.rm)^(1 / r) 
     }
   }
 }
 
 #---- Pythagorean means ----
-mean_arithmetic <- mean_generalized(1)
+arithmetic_mean <- generalized_mean(1)
 
-mean_geometric <- mean_generalized(0)
+geometric_mean <- generalized_mean(0)
 
-mean_harmonic <- mean_generalized(-1)
+harmonic_mean <- generalized_mean(-1)
 
 #---- Extended mean ----
-mean_extended <- function(r, s) {
+extended_mean <- function(r, s) {
   if (!is_number(r) || !is_number(s)) {
-    stop("'r' and 's' must be a finite length 1 numerics")
+    stop("'r' and 's' must be finite length 1 numerics")
   }
   if (small_but_not_zero(r)) {
     warning("'r' is very small in absolute value, but not zero; this can give misleading results")
@@ -62,7 +62,7 @@ mean_extended <- function(r, s) {
   # return function
   function(a, b, tol = .Machine$double.eps^0.5) {
     if (any_negative(a, b)) {
-      warning("Some elements 'a' or 'b' are less than or equal to 0; the extended mean is not defined")
+      warning("some elements of 'a' or 'b' are less than or equal to 0; the extended mean is not defined")
     }
     res <- if (r == 0 && s == 0) {
       sqrt(a * b)
@@ -77,25 +77,47 @@ mean_extended <- function(r, s) {
     }
     # set output to a when a == b
     loc <- which(abs(a - b) <= tol)
-    replace(res, loc, a[wrap_around(a, loc)])
+    res[loc] <- a[wrap_around(a, loc)]
+    res
   }
 }
 
 #---- Logarithmic means ----
-logmean_generalized <- function(r) {
-  mean_extended(r, 1)
+generalized_logmean <- function(r) {
+  extended_mean(r, 1)
 }
 
-logmean <- logmean_generalized(0)
+logmean <- generalized_logmean(0)
 
 #---- Lehmer mean ----
-mean_lehmer <- function(r) {
+lehmer_mean <- function(r) {
   if (!is_number(r)) {
     stop("'r' must be a finite length 1 numeric")
   }
+  # return function
   function(x, w = rep(1, length(x)), na.rm = FALSE) {
-    mean_arithmetic(x, w * x %^% (r - 1), na.rm, scale = TRUE)
+    arithmetic_mean(x, w * x %^% (r - 1), na.rm)
   }
 }
 
-mean_contraharmonic <- mean_lehmer(2)
\ No newline at end of file
+contraharmonic_mean <- lehmer_mean(2)
+
+#---- Nested mean ----
+nested_mean <- function(r, s, t = c(1, 1)) {
+  outer_mean <- generalized_mean(r)
+  if (length(s) != 2) {
+    stop("'s' must be a pair of numeric values")}
+  inner_mean1 <- generalized_mean(s[1])
+  inner_mean2 <- generalized_mean(s[2])
+  if (length(t) != 2 || !is.numeric(t)) {
+    stop("'t' must be a pair of numeric values")
+  }
+  t <- as.numeric(t) # strip any attributes
+  # return function
+  function(x, w1 = rep(1, length(x)), w2 = rep(1, length(x)), na.rm = FALSE) {
+    x <- c(inner_mean1(x, w1, na.rm), inner_mean2(x, w2, na.rm))
+    outer_mean(x, t, na.rm)
+  }
+}
+
+fisher_mean <- nested_mean(0, c(1, -1))
\ No newline at end of file
